/*
  This program is free software published under the
  terms of the GNU Lesser General Public License.
  http://www.gnu.org/copyleft/lesser.html    
  This program is distributed in the hope that it will
  be useful - WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.
*/ 

//   version info:    wm4  version 2.0  updated: 22.03.2019

//  let and const keywords are not used because of backward compatibility

// 11 x var par  => these variables are directly changed by the user within the Options Form interface
var lc = 4;    // minimum (low) count visible shapes
var hc = 8;   // maximum (high) count visible shapes. 
var w = 1;    // number of item within each level between lc..hc
                                // see bellow clock= basic timer resolution in ms  (timer period) =100ms = const
var wait = 6;           // number of basic time cycles before main stimulus (geometric shape) exposition. Units = number of basic timer resolution (clock=100ms). Time of displaying the fixing cross
var time = 1;           // stimulus exposition time in ms. Units = number of basic timer resolution (clock=100ms)
var delay = 10;       // delay between stimulus and user response. Units = number of basic timer resolution (clock=100ms)
var shape = 1;      // 1 = Circle, 3 = Triangle, 4 = Square/Rhomboid, 5 = Polygon-5 sides, 6 = Polygon-6 sides   
var size = 1;         // reverse shape size = (3 + 3 - size); Median = 3. Don't change min and max range without change dependent source code see setVarPar()   
var monochrome= false;   // if true monochrome (black) color circle.  false = circles  will be in random Colors 
var posit = 1;          //  1= full screen;   2 = left-full-right;  3 = quadrants (up-right, down-right,left-down, left-up)  
var isCross=true;       // if isCross=true =>  The cross will appear in the center of the screen before the main stimulus

// global fixed parameters => these variables were not directly available to the user
var clock = 100;      // basic timer resolution in ms  (timer period) => user can't change clock (hardcoded) = const
var itemCnt;    // total item count => itemCnt = w*(hc-lc+1);     
var timerID = null;   // timerID is the identifier of the repeated action you want to cancel. This ID is returned from setInterval(). 
var item = 0;   // current item
var step = -1;   // global timer counter 
var cor = [];   // correct number of visible shapes for each item
var ans = [];  //  user answers for each item (dependent var)
var xy = [];    //  jagged array [item][] Within each [item] is separate array with grid coordinates [0.index= x; 1.index =y, 2.index = show(1) or hide(0) ]
var pos = [];     // screen position for each item (used only if window posit !== 1)
var dirty = false;  // if  (dirty==true) => user changed any options in form
var r;              // shape radius (square base)  
var dx;           // distance between hidden rect (x-coordinate) - x raster
var dy;           // distance between hidden rect (y-coordinate) - y raster


function init() {    
  if (timerID)  {   
    window.clearInterval(timerID);  
  }  
  timerID= setInterval(function () {my_timer()},clock); // Returns an intervalID. intervalID is a unique interval ID you can pass to clearInterval()  
  if (!supportedTransform())  {
    doCenter(document.getElementById("start"));
    doCenter(document.getElementById("response"));
  }    
  document.onkeyup = cancelPScreen;    
  showStart();  
}


function supportedTransform() {
  var prefixes = "transform WebkitTransform MozTransform OTransform msTransform".split(" ");
  var div = document.getElementById("wrapper");
  for(var i = 0; i < prefixes.length; i++) {
    if(div && div.style[prefixes[i]] !== undefined) {
      return prefixes[i];
    }
  }
  return false;
}


function doCenter(e)  {
  if (e.id === "start")  {        
    e.style= (window.innerWidth > 600 && window.innerHeight > 600 ) 
    ? "position:absolute;min-width:50%;min-height:50%;margin:auto;overflow:auto;"  
    : "position:absolute;min-width:75%;min-height:75%;margin:auto;overflow:auto;";  
  }
  else if (e.id === "response")  {
    e.style="position:absolute;min-width:25%;min-height:25%;margin:auto;overflow:auto;";    
  } 
}


// deprecated but still works: https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/keyCode
// not implemented 3/2019: https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/code
// https://caniuse.com/#search=KeyboardEvent.keyCode
// https://caniuse.com/#search=KeyboardEvent.code
// This function is implemented because of possible cheating (we can not prevent the use of cameras on mobile phones)
function cancelPScreen(e)   { 
  e = e || window.e;  
  if (e.key === "PrintScreen" && step >= 0) {      
     e.preventDefault();  
    return false;   
  } 
}

 
// show Instruction panel
function showStart()  {      
  document.getElementById("end").style.display= "none";
  document.getElementById("google-wrapper").style.display="none";  
  document.getElementById("start").style.display= "block";
}


// start research (test)
function startTest()   {      
  if ( (posit === 2 && w %  3 !== 0) || (posit === 3 && w % 4  !== 0))  {                                                                                                   
    var s= "Invalid Number of tasks within each level for selected Position of shapes on the screen. Number of tasks within each level must be ";
    s= (posit === 2 )  ? s + "number must be divisible by 3 without a remainder (3,6,9, ..)" : s + "number must be divisible by 4 without a remainder (4,8,12, ..)";
    document.getElementById("is_valid").textContent= s;    
    document.getElementById("is_valid").setAttribute("class", "error");    
    return;                                                                                         
  } 

  setPar();
  genStim();  
  rndSample(); 
  delInv();   

  if (check()) {
    document.getElementById("is_valid").textContent="";    //setAttribute("class", "hide");   
  }
  else {     
    var e= document.getElementById("is_valid");
    e.textContent="Too small screen resolution. In Options, you need to reduce the Final level and/or increase the Shape size and/or use Full Screen shape position";  
    alert("Too small screen resolution. In Options, you need to reduce the Final level and/or increase the Shape size and/or use Full Screen shape position");  
    e.style="text-align:center;color:red;font-weight:bold;margin: 0 1em;";   //e.className = "error center";        
    return;
  }  
  document.getElementById("footer").style.display= "none";
  document.getElementById("google-wrapper").style.display="none";
  document.getElementById("end").style.display= "none"; 
  document.getElementById("start").style.display= "none"; 
  document.getElementById("canvas").style.display="block";                   
  item=0;
  step=0;   // activate Timer    
}


// set research (paradigm) parameters (They are not available to the user directly)
function setPar()  {   
  itemCnt = w*(hc-lc+1);           
  cor = [];   // correct number of visible shapes within each item
  ans = [];  //  type == string (for google charts) user answers (dependent var)
  xy = [];    //  jagged array [item][] Within each [item] is separate array with grid coordinates [0.index= x; 1.index =y, 2.index = show(1) or hide(0) ]  
  pos= [];   // screen position for each item (used only if window posit !== 1)  
}


// main Stimulus spatial generator => generate x and y coord for shapes ih hidden grid
function genStim()  {     
  var canvas = document.getElementById('canvas');        
  canvas.width= window.innerWidth;       // stretch canvas to parent 
  canvas.height= window.innerHeight;       
  var i= 0;    // ordinal item count [0..itemCnt-1] 
  for (var j=lc; j<=hc; j++)  {  // from low level to high level
    for (var k=1; k<=w; k++)  {    //   from lf to hc (w-times)  
      if (posit ===  1)  {       
        xy[i]= setCoord(0, 0, window.innerWidth, window.innerHeight); // full screen    
      }  
      else if (posit === 2)  { //  left - full screen - right => items will be mixed (shuffle) later; from lf to hc (w-times)  MUST be: (itemCnt %  3 === 0 && w % 3 ===0)
        if ( i % 3 === 0)  {      
          xy[i]=  setCoord(0, 0, window.innerWidth/2, window.innerHeight);  
          pos[i]= 1; // left part of screen
        }
        else if ( i % 3 === 1)  {
          xy[i]= setCoord(0, 0, window.innerWidth, window.innerHeight);     
          pos[i]= 2; // full screen
        }
        else if ( i % 3 === 2)   {   // else
          xy[i]= setCoord(window.innerWidth/2, 0, window.innerWidth, window.innerHeight);         
          pos[i]=3;  // right part of screen         
        }                                                                     
      }      
      else if (posit === 3)    {   //  4 quadrants: up-left, up-right,dow-right, down-left  => items will be mixed (shuffle) later => from lf to hc (w-times) =>  MUST be: (itemCnt %  4 === 0 && w % 4 ===0)            
        switch (i % 4)  {
          case 0:
            xy[i]= setCoord(0, 0, window.innerWidth/2, window.innerHeight/2)   
            pos[i]= 1;  // up-left 
            break;  
          case 1:
            xy[i]= setCoord(window.innerWidth/2, 0, window.innerWidth, window.innerHeight/2);    
            pos[i]= 2;  // up-right
            break; 
          case 2:
            xy[i]= setCoord(window.innerWidth/2, window.innerHeight/2, window.innerWidth, window.innerHeight);   
            pos[i]= 3; // down-right   
            break; 
          case 3:     // else/default
            xy[i]= setCoord(0, window.innerHeight/2, window.innerWidth/2, window.innerHeight);   
            pos[i]= 4; // down-left 
            break; 
        }            
      }                          
      i++;    // next item      
    }  // k= within each level
  }   // j = from low level to high level          
  setRadius(); 
}


// check minimum window.r => shape radius
function check () {
  return (r >= 3);
}


// set item coordinates; generate xy-coord only for one item  => have equidistant (x2-x1) and (y2-y1)
// divide screen section into hc equal segments (x-horizontal and y-vertical) 
// return array (item) xy coordinates
function setCoord(_left, _top, _right, _bottom)   {  
  var arr= [];    //xy[i]= []; => return xy-coord for this item     
  var raster=Math.ceil(Math.sqrt(hc));   // you can set raster to function parameters list
  for (var k=1; k<=raster; k++)   {  // divide line (screen section) into raster equal segments  => divide screen section vertical - y      
    for (var j=1; j<=raster; j++)  {  // divide line (each row) into raster equal segments again => divide screen section horizontal-x          
      var x= divX(_left, _right, -j/(raster-j+1));          // last arg = lambda = -m/n. example if step= 3: -1/3, -2/2, -3/1          
      var y= divY(_top, _bottom, -k/(raster-k+1));    // y= const within each row;  last arg = lambda = -m/n. example if step= 3: -1/3, -2/2, -3/1                       
      arr.push([x, y, 0]);                            
    } // j = horizontal (x) divide
  } // k = vertical (y) divide
  return arr;  
}


// x1 => first point P1 x-coord
// x2 => second point P2 y-coord
// n => (distance(P1-Q)/ distance(Q-P2)) = lambda = -m/n = negative ratio length: P1-Q/Q-P2
// returns: x-coord of Q point
function divX(x1,x2,n)  {   
  return (x1-n*x2)/(1-n);
}


// y1 => first point P1 y-coord
// y2 => second point P2 y-coord
// n  => (distance(P1-Q)/ distance(Q-P2)) = lambda = -m/n = negative ratio length: P1-Q/Q-P2
// returns: y-coord of Q point
function divY(y1,y2,n)  {   
  return (y1-n*y2)/(1-n);
}


// method is safe if (hc===1) but must be called after genStim() and before delInv()
function setRadius()   {
  //  dx = distance between hidden rect (x-coordinate) - x raster, dy =distance between hidden rect (y-coordinate) - y raster
  dx = xy[0][0][0];   // xy[0][0][1]= first shape in 1. row x-coord => all dx = (xy[i][j+1][0] - xy[i][j][0]) = const  
  dy= xy[0][0][1];   // xy[0][0][1]= first shape in 1. row y-coord  => all dy = (xy[i][j+1][1] - xy[i][j][1]) = const     
  r= Math.round(0.5*Math.min(dx,dy)/2);    // if you don't wont (usually vertical y) offset set:  r= Math.floor(Math.min(dx,dy)/2);                                                                                          
  r = Math.round(r/size);   // size => reverse shape size (3 + 3 - size). median value in wm4.html is 3. recode values: 5=>1,4=>2,3=>3,2=>4,1=>5            
}


function rndSample()  {    
  var tmp=[];      // already choosen items 
  var _item= 0;        // ordinal item count [0..itemCnt-1]
  for (var cnt=lc; cnt<=hc; cnt++)  
  {  //cnt = number of visible shapes within this item
    for (var j=0; j<w; j++)  
    {    //  from lf to hc (w-times) => choose cnt random shapes within this item                 
      for (var i=0; i<cnt; i++)  {
        do  
          var rnd =  Math.floor(Math.random()*xy[_item].length);    
        while (xy[_item][rnd][2] === 1);   // check if already choosen
        xy[_item][rnd][2]=1;       // this circle will be visible
      }
      _item++;     
    }
  }
  xy= shuffle(xy);
  for (var i=0; i<itemCnt; i++)  { 
    cor[i]=0;  // correct number of visible shapes within each item
    for (var j in xy[i]) {                   
      if (xy[i][j][2] === 1) cor[i]++;  
    }    
  }
}


// unbiased shuffle algorithm: Fisher-Yates (aka Knuth) Shuffle.
function shuffle(arr)   {
  var cur_index = arr.length, tmp, rnd_index, tmp2;
  // While there remain elements to shuffle...
  while (0 !== cur_index)   {    
    rnd_index = Math.floor(Math.random() * cur_index);     // Pick a remaining element...
    cur_index = cur_index - 1;   
    tmp = arr[cur_index];          // And swap it with the current element.
    if (posit !== 1)  tmp2= pos[cur_index];      //  also do not forget swap screen positions
    arr[cur_index] = arr[rnd_index];
    arr[rnd_index] = tmp;     
    if (posit !== 1)  {    //  swap screen positions
      pos[cur_index] = pos[rnd_index];
      pos[rnd_index] = tmp2;       
    }
  }
  return arr;
}


// delete (unset) all invisible shapes because of better future performance in drawGrid() method
function delInv()  {   
  for (var i in xy)  {  
    for (var j in xy[i]) {                   
      if (xy[i][j][2] !== 1)  delete(xy[i][j]);  // this works OK. tested with simple counter in drawGrid()    
    }
  }
}


function getRndInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1) ) + min;
}


function drawCross()  {
  var canvas= document.getElementById("canvas");   
  var x= Math.round(canvas.width/2);
  var y= Math.round(canvas.height/2);
  var d = Math.floor(Math.min(x,y)/4);  // 1/8 min(width, height)  
  drawLine(x-d,y,x+d,y); // horizontal
  drawLine(x,y-d,x,y+d); // vertical 
}


function drawLine(x1,y1,x2,y2)  {
  var ctx= document.getElementById("canvas").getContext('2d'); 
  ctx.beginPath();
  ctx.moveTo(x1,y1);
  ctx.lineTo(x2,y2);
  ctx.lineWidth= 1;
  ctx.strokeStyle="#000000";  // black
  ctx.stroke();     
}


function drawGrid()  {  
  var ctx = document.getElementById("canvas").getContext('2d'); 
  ctx.clearRect(0, 0, canvas.width, canvas.height);      // clear canvas             
  var m_x_off= Math.floor(dx/2-r); // maximum x-offset within hidden rect
  var m_y_off= Math.floor(dy/2-r); // maximum y-offset within hidden rect
  for (var j in xy[item])   {                                    
      
      var x_off= getRndInt(-m_x_off,m_x_off); // random x-offset within hidden rect
      var y_off= getRndInt(-m_y_off,m_y_off); // random y-offset within hidden rect
      ctx.beginPath();                    
      if (shape === 1) {   // draw circle
        ctx.arc(xy[item][j][0]+x_off, xy[item][j][1]+y_off, r, 0, 2*Math.PI);     //ctx.arc(x,y,r,sAngle,eAngle,counterclockwise);   
      }
      else  {  // draw regular polygon (3,4,5,6 sides)
        var x= xy[item][j][0]+x_off;
        var y= xy[item][j][1]+y_off;
        for (var j= 1;  j<=shape; j++)  {            
          var px = x + r * Math.cos(j*2*Math.PI/shape);
          var py = y + r * Math.sin(j*2*Math.PI/shape);
          (j===1) ? ctx.moveTo(px,py) : ctx.lineTo(px,py);                     
        }    
      }                           
      ctx.fillStyle = (monochrome==true)  ? ctx.fillStyle =  "black" :   getRandomColor();       //ctx.fillStyle =  (xy[item][i][2] == 0) ?  "black" : "red";    //  getRandomColor();          
      ctx.fill();                                  
  }                  
}


function getRandomColor() {
  var letters = '0123456789ABCDEF';
  var color = '#';
  for (var i = 0; i < 6; i++) {
    color += letters[Math.floor(Math.random() * 16)];     // returns a random integer from 0 to 15 
  }
  return color;
}  


function my_timer()  {     
   var canvas= document.getElementById("canvas");      
   if (step<0) return;  // nop, idle
   step++; 
   if (step < wait-1)  {   // time before main stimulus (points) period
      if (!isCross) return;   // idle, just wait main stimulus
      if (step % 2 !== 0)  {     // always trigger this in first timer call
        drawCross();     // Draw Cross only if it is odd number  (1,3,5,..)  example if (before==6) = 3xflick    
      }
      else   {       // for even numbers  (2,4,6,..) => clear previous cross
        canvas.getContext("2d").clearRect(0, 0, canvas.width, canvas.height);     // clear canvas => clear Cross in 2,4,6 step         
      }        
   }
   else if (step === wait-1)   {       // ALWAYS clear canvas before main stimulus display
     if (!isCross) return;   // idle, just wait main stimulus
     canvas.getContext("2d").clearRect(0, 0, canvas.width, canvas.height);      // clear last cross if exists                
   }    
   else if (step === wait) {
     drawGrid();     // draw main stimulus
   }
   else if (step === wait+time) {
     canvas.getContext("2d").clearRect(0, 0, canvas.width, canvas.height);      // clear main stimulus 
   }
   else if (step >= wait+time+delay) {
      step = -1;         // nop, idle, do none, Timer no operation
      showResponse();    
   }
}


function keyUp(event)  {          
  var e = event ||  window.event;    // event = use the value of event if available or   window.event = if not assume it's IE and use window.event   
  e.preventDefault();            
  if (e.keyCode === 13) {   
    nextItem();         
  }
}


// goto next item/task or end research
function nextItem()  {
  var node= document.getElementById("answer");    
  if (!node.validity.valid) {
    document.getElementById("answer_val").textContent= "Enter the number of between 0 and 99. If you do not know the answer: enter zero or empty (blank)."; 
    document.getElementById("answer").focus();   //alert("Enter the number of between 0 and 99. If you do not know the answer: enter zero or empty (blank).");
    return;
  }   
  document.getElementById("answer_val").textContent="";  
  
  var tmp = (trim(node.value) == "") ? 0: node.value;
  tmp = Math.round(parseInt(tmp,10));         
  ans[item]= tmp;     // store validated user answer (as integer)
  node.value= "";          // reset user answer (default answer for next item)
  node= document.getElementById("response");
  node.style.display="none";            
  if (item < xy.length - 1) {    // show next task        
    document.getElementById("canvas").style.display="block"; 
    item++;
    step=0;      // activate Timer    
  }
  else  {                
    document.getElementById("response").style.display="none";
    document.getElementById("end").style.display="block";  
    theEnd();    
  }
}


function trim(str) {
   return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
}


// show response div (panel with user answer)
function showResponse()  {
  document.getElementById("canvas").style.display="none";
  var node= document.getElementById("response").style.display="block";  
  document.getElementById("answer").focus(); 
}


// the end of research  => prepare data and calls google Chart API for report
function theEnd()  {
  var res=0;  
  for (var j=0; j!=cor.length; j++)   {
    if (cor[j] === ans[j]) res++;
  }
  var elem= document.getElementById("score");
  elem.textContent="Your score is: " + res + "   from possible: " + cor.length + "   Percentage: " + Math.round(100*res/cor.length) + "%";
  document.getElementById("google-wrapper").style.display="block"; 
  document.getElementById("table1").innerHTML="";  
  document.getElementById("table2").innerHTML="";   
  document.getElementById("chart1").innerHTML=""; 
  document.getElementById("chart2").innerHTML="";   
  document.getElementById("no_google").textContent="";    
  try  {
    showReport1(res);
    if (posit > 1)  showReport2(res); 
  }
  catch (e)  {
    document.getElementById("no_google").textContent= "You must have an available internet connection to display graphs and tables. Make sure you are online."; 
  }             
}


// prepare data for google API chart/table for table1 (and chart1) 
// table1 = Displays the answer (and results) for each item.
// chart1 = Show correct answers grouped per level
function showReport1(res)   {      
  //    xxxxxxxxxxxxxx     table1 = Displays the answer (and results) for each item        xxxxxxxxxxxxxxxxx 
  var data = new google.visualization.DataTable();    // Create and populate the data table. 
  data.addColumn('string', 'Item number');     // 1. col = ans array = must be string  
  if (posit > 1)  data.addColumn('string', 'Screen position'); 
  data.addColumn('number', 'Answer');         
  data.addColumn('number', 'Correct'); 
  data.addColumn('number', 'Item score');    
  for(var i = 0; i < ans.length; i++)  {  
    var tmp= (ans[i] === cor[i]) ? 1: 0;    
    if (posit === 1)  {
      data.addRow([(i+1).toString() + ". item",  ans[i],  cor[i], tmp]);        
    }
    else if (posit === 2)  {
       switch (pos[i])  {
         case 1:           
           data.addRow([(i+1).toString() + ". item", "left", ans[i],  cor[i], tmp]);
           break;           
         case 2:  
           data.addRow([(i+1).toString() + ". item",  "full screen", ans[i],  cor[i], tmp]);
           break;      
         case 3:  
           data.addRow([(i+1).toString() + ". item",  "right", ans[i],  cor[i], tmp]);
       }           
    }
    else if (posit === 3)  {
      switch (pos[i])  {
        case 1:  
          data.addRow([(i+1).toString() + ". item",  "top-left", ans[i],  cor[i], tmp]);
          break;
        case 2:
          data.addRow([(i+1).toString() + ". item",  "top-right", ans[i],  cor[i], tmp]); 
          break;
        case 3:  
          data.addRow([(i+1).toString() + ". item",  "down-right", ans[i],  cor[i], tmp]);
          break;
        case  4:  
          data.addRow([(i+1).toString() + ". item",  "down-left", ans[i],  cor[i], tmp]);
      }
    }                 
  }                                   
  options= {      
    /*'width': '50%',*/    
    'allowHtml': true, 
    'sort': 'enable',  
    'alternatingRowStyle': true              
  };           
  var elem= document.getElementById("table1");
  elem.title="Total score=" + res;     // see wm4.html some title are static!!
  elem.style.display = "block"; 
  new google.visualization.Table(elem).draw(data, options);  
  //    xxxxxxxxxxx     chart1 => BarChart => Show correct answers grouped per level  xxxxxxxxxxxxx
  var lev = {};  // correct answers for each level lc..hc  (group by level)
  for(var i= lc; i<= hc; i++)  {  
    lev[i]= 0; 
  }  
  for (var j=0; j!=cor.length; j++)   {
    if (cor[j] === ans[j]) lev[cor[j]]++;
  }    
  var data = new google.visualization.DataTable();    // create and populate new data table. 
  data.addColumn('string', 'Shapes count');     // 1. col = levels [lc..hc] = must be string             
  data.addColumn('number', 'Score within the level'); 
  data.addColumn('number', 'Maximum possible score');
  /*for (var key in lev)  {
     data.addRow([(key).toString() + ". Shapes",  lev[key],  window.w]);                                            
  } */ 
  // https://stackoverflow.com/questions/54327508/how-to-display-a-small-bar-even-when-a-value-is-zero-in-google-charts
  for (var key in lev)  {
    if (lev[key] > 0) { 
      data.addRow([(key).toString() + " shapes",  lev[key],  window.w]); 
    }
    else  { 
      data.addRow([(key).toString() + " shapes",   {v: 0.1, f: '0'},  window.w]);   
    }                                       
  }   
  // https://developers.google.com/chart/interactive/docs/gallery/columnchart              
  options= {
    title: 'The results are grouped by the number of displayed shapes. Total score = ' + res,
    width: '100%',  
    hAxis: {
      title: 'Number of displayed geometric shape',
    }, 
    vAxis: {
      title: 'Number of points'
    }   
  };    
  document.getElementById("chart1").style.display = "block";   
  new google.visualization.ColumnChart(document.getElementById('chart1')).draw(data, options);     // Create and draw the visualization.      
}


// prepare data and call google API chart/table for table2 (and chart2) 
// table2 and chart2 = Displays the sum of correct answers grouped by screen positions 
function showReport2(res) {    
  var s_pos = {};  // sum of correct answers group for each screen position (1..3) or (1..4)
  var up= (posit === 2) ? 3 : 4;   // (window.posit === 2) => [0..3] else [0..4] see genStim()
  // s_pos[1] = left or up-left; s_pos[2]= full screen or up-right, s_pos[3] = right or down-right, s_pos[4]= down-left
  for(var i= 1; i<=up; i++)  {  
    s_pos[i]= 0;        
  }  
  for (var j=0; j!=cor.length; j++)   {         
    if (cor[j] === ans[j])  s_pos[pos[j]]++;       
  }    
  //    xxxxxxxxxxxxxx     table2 = Displays the sum of correct answers grouped by screen positions  xxxxxxxxxxxxxxxxx 
  var data = new google.visualization.DataTable();    // Create and populate the data table. 
  data.addColumn('string', 'Screen position');     // 1. col = ans array = must be string  
  data.addColumn('number', 'The sum of points');    // sum of correct answers grouped by screen positions      
  // we must recode pos[] and s_pos[] values based on window.posit => see genStim() 
  for (var i in s_pos)  {  
     if (posit === 2)  {
       switch (i)  {
         case "1":           
           data.addRow(["left", s_pos[i]]);
           break;           
         case "2":  
           data.addRow(["full screen", s_pos[i]]);
           break;      
         case "3":  
           data.addRow(["right", s_pos[i]]);
       }           
    }
    else if (posit === 3)  {
      switch (i)  {
        case "1":  
          data.addRow(["top-left", s_pos[i]]);
          break;
        case "2":
          data.addRow(["top-right", s_pos[i]]);
          break;
        case "3": 
          data.addRow(["down-right", s_pos[i]]); 
          break;
        case  "4": 
          data.addRow(["down-left", s_pos[i]]); 
      }
    }                   
  }                                    
  options= {      
    'width': '50%',    
    'allowHtml': true, 
    'sort': 'enable',  
    'alternatingRowStyle': true              
  };           
  var elem= document.getElementById("table2");
  elem.style.display = "block"; 
  new google.visualization.Table(elem).draw(data, options); 
  //    xxxxxx      PIE chart2  => data are same   xxxxxxx
  if (res<1)  { 
    document.getElementById("chart2").innerHTML= '<p class="center">Total score = 0. The Chart will not be displayed';
    return;
  }
  options= {
   title: 'Sum of points grouped by the display position of geometric shapes on the screen',
   is3D: true,
   width: '100%'   
  };    
  document.getElementById("chart2").style.display = "block";   
  new google.visualization.PieChart(document.getElementById('chart2')).draw(data, options);     // Create and draw the visualization.     
}


// refresh variable research Options (update *_msg label) => give user short option HELP
// This method too warms the CPU (and the world's oceans). In the future it should be optimized
function refOption(e)  {   
  dirty=true;       
  var my_id = e.id, msg = document.getElementById(my_id + "_msg"), s = "Values within this range are allowed: ", s2= " with step= ";    
  if ( my_id === "lc")  {
    if (e.validity.valid)  {           
      msg.removeAttribute("class");       //msg.textContent=  e.value; 
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
      msg.setAttribute("class", "error");    // msg.className = 'error';
    } 
    refItem();
  }      
  else if ( my_id === "hc")  {
    if (e.validity.valid)  {     
      msg.removeAttribute("class");           //   msg.textContent=  e.value;
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step; 
      msg.setAttribute("class", "error");
    }  
    refItem();
  }      
  else if ( my_id === "w")  {    // Check interdependent with posit
    if (e.validity.valid)  {       
      msg.removeAttribute("class");        
      msg.textContent= "";        //msg.textContent=  e.value; 
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
      msg.setAttribute("class", "error");    // msg.className = 'error';
    }     
    var _pos= document.getElementById("posit");
    if (_pos.value === "2" && e.value %  3 != 0)  {
      msg.textContent= "For selected Screen shape position Number of tasks within each level must be divisible by 3 with no remainder (3,6,9,..)";
      msg.setAttribute("class", "error");  
    }
    else if (_pos.value === "3" && e.value %  4 != 0)   {
      msg.textContent= "For selected Screen shape position Number of tasks within each level must be divisible by 4 with no remainder (4,8,12,..)";
      msg.setAttribute("class", "error");  
    }           
    refItem();
  }   
  else if ( my_id === "wait")  {
    if (e.validity.valid)  {
      msg.textContent=  e.value + " ms"; 
      msg.removeAttribute("class");
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step; 
      msg.setAttribute("class", "error");
    }          
  }    
  else if ( my_id === "time")  {
    if (e.validity.valid) {
      msg.textContent= e.value + " ms"; 
      msg.removeAttribute("class");
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
      msg.setAttribute("class", "error");
    } 
  }    
  else if ( my_id === "delay")  {
    if ( e.validity.valid ) {
      msg.textContent= e.value + " ms";
      msg.removeAttribute("class");
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
      msg.setAttribute("class", "error");
    }
  }                  
  else if ( my_id === "size")  {
    if ( e.validity.valid ) {                     
      msg.textContent= "Significantly depends on the final level. The higher it is, the size of the dot is smaller and vice versa"; 
      msg.removeAttribute("class");            
    }
    else  {
      msg.textContent= s + e.min + ".." + e.max + s2 + e.step;
      msg.setAttribute("class", "error");
    }
  }    
  else if (my_id === "monochrome")  {                                                                                                                                               
    msg.textContent= (e.value === "1") ? "Geometric shapes will only appear in black color- monochrome" : "Geometric shapes will be displayed in all colors (random)";
  }  
  else if (my_id === "posit")  {    // Check interdependent with w
    var w_msg=  document.getElementById("w_msg");    // validator for window.w
    w_msg.textContent= "";
    w_msg.removeAttribute("class");      
    if (e.value === "1")  {
      msg.textContent=  "Geometric shapes will be displayed on the full screen";      
    }             
    else if (e.value === "2")   {
      msg.textContent= "1/3 of the geometric shapes will be displayed on the left half of the screen, 1/3 on the full screen, 1/3 on the right half of the screen.";
      if (parseInt(document.getElementById("w").value, 10)  %  3 !== 0 )  {
          w_msg.textContent= "For selected Screen shape position Number of tasks within each level must be divisible by 3 with no remainder (3,6,9,..)";
          w_msg.setAttribute("class", "error");
       }
    }
    if (e.value === "3")  {
      msg.textContent= "1/4 geometric shapes will be shown top-left,1/4 of the shapes will be shown top-right, 1/4  will be shown down-left, 1/4 will be shown down-right."; 
      if (parseInt(document.getElementById("w").value, 10)  %  4 !== 0 )  {
        w_msg.textContent= "For selected Screen shape position Number of tasks within each level must be divisible by 3 with no remainder (4,8,12,..)";
        w_msg.setAttribute("class", "error");
      }
    }
  }          
  refMsg();   
}

// refresh main validator message for user defined OPTIONS (feedback to user)
function refMsg()   { 
 if ( optionsValid() )   {
    document.getElementById("is_valid").textContent=  (dirty===true) ? "All your settings have been successfully updated" : "";  
    document.getElementById("is_valid").style="text-align:center;color:#000000"; 
  }
  else  {
    document.getElementById("is_valid").textContent="Some of your options are incorrectly set. They were not updated. These settings are marked in red."; 
    document.getElementById("is_valid").style= "text-align:center;color:#ff0000;font-weight:bold"; // safari 5.1.7 bug? => this don't work
  } 
}


// update item count on options form and check lc <= hc and update  label itemCnt = w*(hc-lc+1);
function refItem()  {
   var _lc = document.getElementById("lc");
   var _hc = document.getElementById("hc");
   var _w= document.getElementById("w");
   var _item= document.getElementById("item_cnt");
   var lc_msg= document.getElementById("lc_msg");
   var hc_msg= document.getElementById("hc_msg");    
   if (_lc.validity.valid  && _hc.validity.valid && _w.validity.valid)   
   {                      
     if  (parseInt(_lc.value,10) > parseInt(_hc.value,10))  
     {
       lc_msg.textContent= "The starting level must not be higher than the final level";  
       hc_msg.textContent= "The starting level must not be higher than the final level";
       lc_msg.setAttribute("class", "error");
       hc_msg.setAttribute("class", "error");    
       _item.textContent= "Total number of items: Invalid value"; 
       _item.setAttribute("class", "error");
     } 
     else   {                                                                                               
       lc_msg.removeAttribute("class");    //lc_msg.textContent= _lc.value;
       hc_msg.removeAttribute("class");   //hc_msg.textContent= _hc.value; 
       _item.textContent=  "Total number of items: " +  ( parseInt(_w.value,10) * (parseInt(_hc.value,10)-parseInt(_lc.value,10)+1));      
       _item.removeAttribute("class");
     }
   } 
   else  
   {
     _item.textContent= "Total number of items: Invalid value"; 
     _item.setAttribute("class", "error");
   }    
}


// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
function showOption()   {
  document.getElementById("hide_option").style.display ="block";
  document.getElementById("options").style.display= "block";
  document.getElementById("start").classList.remove("child");  // don't work in <= IE 9    
  document.getElementById("start_div").style.display="none";  // .disabled = true;   
  document.getElementById("show_option").style.display ="none";  
  document.getElementById("hide_option").style="text-align:center";  // bug? auto-remove style when element.style.display="none"
  document.getElementById("instructions").style.display ="none"; 
  document.getElementById("footer").style.display="none"; 
  dirty= false;
}


// <!-- IMPORTANT: showOption() and hideOption() buttons must be outside form element -->
//  save to window global vars all user defined research Options
function hideOption()  {
  setVarPar();        
  document.getElementById("options").style.display="none"; 
  document.getElementById("start").classList.add("child");   
  document.getElementById("show_option").style.display ="block";   
  document.getElementById("start_div").style.display="block";        //.removeAttribute("disabled");  
  document.getElementById("hide_option").style.display ="none";   
  document.getElementById("start_div").style="text-align:center";  // bug?  (firefox auto remove class "center" when set style.display.none;)
  document.getElementById("show_option").style="text-align:center";   // bug?
  document.getElementById("instructions").style.display= "block"; 
  refMsg() ;    
}


// set user variable parameters; don't use this code in setPar() => eq: document.getElementById("time").value are not updated to real value!!  
function setVarPar()  {   
  if (document.getElementById("lc").validity.valid && document.getElementById("hc").validity.valid 
    && (parseInt(document.getElementById("lc").value,10) <= parseInt(document.getElementById("hc").value, 10) ) )   {  
      lc= parseInt(document.getElementById("lc").value, 10); 
      hc= parseInt(document.getElementById("hc").value, 10); 
  }
  if (document.getElementById("w").validity.valid) window.w= parseInt(document.getElementById("w").value, 10);        
  if (document.getElementById("wait").validity.valid) window.wait= Math.round(parseInt(document.getElementById("wait").value, 10)/clock);
  if (document.getElementById("time").validity.valid) window.time= Math.round(parseInt(document.getElementById("time").value, 10)/clock);  
  if (document.getElementById("delay").validity.valid)  window.delay= Math.round(parseInt(document.getElementById("delay").value, 10)/clock);     
  if (document.getElementById("shape").validity.valid) shape = parseInt(document.getElementById("shape").value,10);
  if (document.getElementById("size").validity.valid) {
    size = parseInt(document.getElementById("size").value, 10);
    size =  (3 + 3 - size); // size => reverse shape size (3 + 3 - size). median value in wm4.html is 3. recode values: 5=>1,4=>2,3=>3,2=>4,1=>5     
  }
  monochrome= (document.getElementById("monochrome").value === "1") ? true: false;  // if true monochrome (only black) color circle.  false = circles  will be in random Colors  
  posit=  parseInt(document.getElementById("posit").value, 10);     // shapes positions on screen
  var e=   document.getElementById("posit");
  var s = "Warning:  Screen shape position: " + e.options[e.selectedIndex].textContent;
  if (posit === 2 && w %  3 !== 0)   {     
    window.w= 3;             // Automatic set to default values
    document.getElementById("w").value= w;   // wm4.html refresh/update
    s= s + " must have Number of tasks within each level must be divisible by 3 with no remainder (3,6,9,..). Set to default value 3. This change also includes changing the number of items in the task";
    document.getElementById("is_valid").textContent= s;       
    document.getElementById("is_valid").setAttribute("class", "error");                                                                                      
  }
  else if  (posit === 3 && w % 4  !== 0)   {   
    window.w= 4;      // Automatic set to default values
    document.getElementById("w").value= w;   // wm4.html refresh/update
    s = s +  " must have Number of tasks within each level must be divisible by 4 with no remainder (4,8,12,..). Set to default value 4. This change also includes changing the number of items in the task";    
    document.getElementById("is_valid").textContent= s;      
    document.getElementById("is_valid").setAttribute("class", "error");                                                                                     
  } 
  isCross= (document.getElementById("is_cross").value === "1") ? true: false;   // if (isCross==true) => The cross will appear in the center of the screen before the main stimulus        
  document.getElementById("p_cross").style.display= (isCross) ? "block": "none";  // refresh user instructions see wm4.html 
}


// setVarPar() is called before this function  => set default w values  if (!w_ok) => w=3 || w=4
function optionsValid()  {
  var range_ok= parseInt(document.getElementById("lc").value,10) <= parseInt(document.getElementById("hc").value,10);    
  var w_ok = !((document.getElementById("posit").value === "2" && parseInt(document.getElementById("w").value, 10) %  3 !== 0) || ( document.getElementById("posit").value === "3" && parseInt(document.getElementById("w").value,10) % 4  !== 0));  
  return (      
    document.getElementById("lc").validity.valid &&  
    document.getElementById("hc").validity.valid && 
    document.getElementById("w").validity.valid  &&  
    document.getElementById("wait").validity.valid && 
    document.getElementById("time").validity.valid &&       
    document.getElementById("delay").validity.valid &&
    document.getElementById("shape").validity.valid && 
    range_ok  &&  w_ok);                    
}
